﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Web;
using System.Xml;
using System.Xml.Linq;

using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Integration;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Ecommerce.Prices;
using Dynamicweb.Environment;
using Dynamicweb.Ecommerce.Products.Taxes;
using Dynamicweb.Data;
using Dynamicweb.Core;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    internal class OrderHandler
    {
        private static CacheLevel OrderCacheLevel
        {
            get
            {
                string cacheLevelString = Helpers.GetConfigValue(Constants.CacheConfiguration.Path, Constants.CacheConfiguration.Order);
                return Helpers.GetEnumValueFromString<CacheLevel>(cacheLevelString, CacheLevel.Page);
            }
        }

        private static bool AddOrderFieldsToRequest
        {
            get
            {
                return Convert.ToBoolean(Helpers.GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AddOrderFieldsToRequest));
            }
        }

        private static bool AddOrderLineFieldsToRequest
        {
            get
            {
                return Convert.ToBoolean(Helpers.GetConfigValue(Constants.CustomFieldsSettings.Path, Constants.CustomFieldsSettings.AddOrderLineFieldsToRequest));
            }
        }

        public static bool UpdateOrder(Order order)
        {
            if (order == null)
                return false;

            if (ExecutingContext.IsFrontEnd() && !Global.IsIntegrationForAnonymousUsersAllowed && User.GetCurrentExtranetUser() == null 
                && User.GetUserByID(order.CustomerAccessUserId) == null)
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, "No user is currently logged in. Anonymous user is not allowed.");
                return false;
            }

            /* create order: if it is false, you will get a calculate order from nav with the total prices */
            /* if it is true, then a new order will be created in NAV */
            bool createOrder = order.Complete;

            /* Create order if the request is from Backend */
            if (ExecutingContext.IsBackEnd() && !createOrder)
            {
                createOrder = true;
            }

            string orderIdentifier = Helpers.OrderIdentifier(order);

            Dictionary<string, XmlDocument> responsesCache = ErpResponseCache.GetWebOrdersConnectorResponses(OrderCacheLevel);

            XmlDocument response = null;
            if (!createOrder && responsesCache.ContainsKey(orderIdentifier))
            {
                response = responsesCache[orderIdentifier];
            }
            else
            {
                XmlDocument request = BuildOrderRequest(order, createOrder);
                response = Connector.CalculateOrder(request.OuterXml);
                if (responsesCache.ContainsKey(orderIdentifier))
                {
                    responsesCache.Remove(orderIdentifier);
                }
                if (response != null && !string.IsNullOrEmpty(response.InnerXml))
                {
                    responsesCache.Add(orderIdentifier, response);
                }
            }
            if (response != null && !string.IsNullOrEmpty(response.InnerXml))
            {
                ProcessResponse(response, order, createOrder);
            }
            else
            {
                //error occured
                return false;
            }
            return true;
        }

        private static XmlDocument BuildOrderRequest(Order order, bool createOrder)
        {
            // Create XmlDocument object with Declaration
            XmlDocument xDoc = new XmlDocument();
            XmlDeclaration xDec = xDoc.CreateXmlDeclaration("1.0", "UTF-8", "yes");
            xDoc.AppendChild(xDec);
            // Create Request root node
            XmlElement xRoot = xDoc.CreateElement("CalculateOrder");
            XmlElement ordersNode = xDoc.CreateElement("Orders");

            XmlElement orderNode = xDoc.CreateElement("Order");

            if (ExecutingContext.IsBackEnd())
            {
                //resend order from backend
                Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerAccessUserExternalId", Helpers.GetCustomerIdentifier(User.GetUserByID(order.CustomerAccessUserId)));
            }
            else
            {                
                Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerAccessUserExternalId", 
                    User.GetCurrentExtranetUser() == null ? Helpers.GetCustomerIdentifier(User.GetUserByID(order.CustomerAccessUserId)) : Helpers.CustomerIdentifier);
            }

            /* if it is true, then a new order will be create in NAV */
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "CreateOrder", createOrder.ToString());
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderID", order.Id);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderIntegrationOrderID", order.IntegrationOrderId);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCurrencyCode", order.CurrencyCode);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDate", order.Date.ToString());
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderPaymentMethodName", order.PaymentMethod);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderShippingMethodName", order.ShippingMethod);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerName",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerName")) ? Context.Current.Request.GetString("EcomOrderCustomerName") : order.CustomerName);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerAddress",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerAddress")) ? Context.Current.Request.GetString("EcomOrderCustomerAddress") : order.CustomerAddress);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerAddress2",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerAddress2")) ? Context.Current.Request.GetString("EcomOrderCustomerAddress2") : order.CustomerAddress2);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerCity",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerCity")) ? Context.Current.Request.GetString("EcomOrderCustomerCity") : order.CustomerCity);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerCountryCode",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerCountry")) ? Context.Current.Request.GetString("EcomOrderCustomerCountry") : order.CustomerCountryCode);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerEmail",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerEmail")) ? Context.Current.Request.GetString("EcomOrderCustomerEmail") : order.CustomerEmail);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerFax",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerFax")) ? Context.Current.Request.GetString("EcomOrderCustomerFax") : order.CustomerFax);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerPhone",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerPhone")) ? Context.Current.Request.GetString("EcomOrderCustomerPhone") : order.CustomerPhone);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerZip",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerZip")) ? Context.Current.Request.GetString("EcomOrderCustomerZip") : order.CustomerZip);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryName",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryName")) ? Context.Current.Request.GetString("EcomOrderDeliveryName") : order.DeliveryName);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryAddress",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryAddress")) ? Context.Current.Request.GetString("EcomOrderDeliveryAddress") : order.DeliveryAddress);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryAddress2",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryAddress2")) ? Context.Current.Request.GetString("EcomOrderDeliveryAddress2") : order.DeliveryAddress2);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryCity",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryCity")) ? Context.Current.Request.GetString("EcomOrderDeliveryCity") : order.DeliveryCity);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryCountryCode",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryCountry")) ? Context.Current.Request.GetString("EcomOrderDeliveryCountry") : order.DeliveryCountryCode);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryEmail",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryEmail")) ? Context.Current.Request.GetString("EcomOrderDeliveryEmail") : order.DeliveryEmail);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryFax",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryFax")) ? Context.Current.Request.GetString("EcomOrderDeliveryFax") : order.DeliveryFax);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryPhone",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryPhone")) ? Context.Current.Request.GetString("EcomOrderDeliveryPhone") : order.DeliveryPhone);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryZip",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryZip")) ? Context.Current.Request.GetString("EcomOrderDeliveryZip") : order.DeliveryZip);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderCustomerCompany",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderCustomerCompany")) ? Context.Current.Request.GetString("EcomOrderCustomerCompany") : order.CustomerCompany);
            Helpers.AddChildXmlNodeToCart(xDoc, orderNode, "OrderDeliveryCompany",
                !string.IsNullOrEmpty(Context.Current.Request.GetString("EcomOrderDeliveryCompany")) ? Context.Current.Request.GetString("EcomOrderDeliveryCompany") : order.DeliveryCompany);

            if (AddOrderFieldsToRequest && order.OrderFieldValues.Count > 0)
            {
                XmlElement fields = xDoc.CreateElement("Fields");
                foreach (OrderFieldValue ofv in order.OrderFieldValues)
                {
                    Helpers.AddChildXmlNodeToCart(xDoc, fields, ofv.OrderField.SystemName, ofv.Value != null ? ofv.Value.ToString() : string.Empty);
                }
                orderNode.AppendChild(fields);
            }
            XmlElement orderLinesNode = xDoc.CreateElement("OrderLines");
            foreach (OrderLine ol in order.OrderLines)
            {
                if (ol.Type != "1" && ol.Type != "3" && ol.Type != "4")//1, 3 - Discount type; 4 - Tax type
                {
                    XmlElement orderlineNode = xDoc.CreateElement("OrderLine");

                    Helpers.AddChildXmlNodeToCart(xDoc, orderlineNode, "OrderLineID", ol.Id);
                    Helpers.AddChildXmlNodeToCart(xDoc, orderlineNode, "OrderLineProductID", ol.ProductNumber);
                    Helpers.AddChildXmlNodeToCart(xDoc, orderlineNode, "OrderLineQuantity", ol.Quantity.ToString(CultureInfo.GetCultureInfo("en-GB")));
                    Helpers.AddChildXmlNodeToCart(xDoc, orderlineNode, "OrderLinePriceWithoutVAT",
                                                  ol.Price.PriceWithoutVAT.ToString());
                    Helpers.AddChildXmlNodeToCart(xDoc, orderlineNode, "OrderLineUnitPriceWithoutVAT",
                                                  ol.UnitPrice.PriceWithoutVAT.ToString());
                    Helpers.AddChildXmlNodeToCart(xDoc, orderlineNode, "OrderLineUnitID", ol.UnitId ?? string.Empty);

                    if (AddOrderLineFieldsToRequest && ol.OrderLineFieldValues.Count > 0)
                    {
                        XmlElement olFields = xDoc.CreateElement("OrderLineFields");
                        foreach (OrderLineFieldValue olfv in ol.OrderLineFieldValues)
                        {
                            Helpers.AddChildXmlNodeToCart(xDoc, olFields, olfv.OrderLineFieldSystemName, olfv.Value != null ? olfv.Value.ToString() : string.Empty);
                        }
                        orderlineNode.AppendChild(olFields);
                    }
                    orderLinesNode.AppendChild(orderlineNode);
                }
            }
            ordersNode.AppendChild(orderNode);

            xRoot.AppendChild(ordersNode);
            xRoot.AppendChild(orderLinesNode);
            xDoc.AppendChild(xRoot);
            return xDoc;
        }

        private static void ProcessResponse(XmlDocument response, Order order, bool createOrder)
        {
            if (response == null || order == null)
            {
                if (createOrder)
                {
                    Logger.Instance.Log(ErrorLevel.Error, "Response CreateOrder is null.");
                }
                return;
            }

            try
            {
                XmlNode orderNode = response.SelectSingleNode("//item [@table='EcomOrders']");
                // Set Order prices
                order.AllowOverridePrices = true;
                try
                {
                    order.Price.Currency = Helpers.CurrentCurrency;
                    order.Price.PriceWithVAT = Helpers.ToDouble(orderNode.SelectSingleNode("column [@columnName='OrderPriceWithVAT']").InnerText);
                    order.PriceBeforeFees.PriceWithVAT = order.Price.PriceWithVAT;

                    order.Price.VAT = Helpers.ToDouble(orderNode.SelectSingleNode("column [@columnName='OrderPriceVAT']").InnerText);
                    order.PriceBeforeFees.VAT = order.Price.VAT;

                    order.Price.PriceWithoutVAT = order.Price.PriceWithVAT - order.Price.VAT;
                    order.PriceBeforeFees.PriceWithoutVAT = order.Price.PriceWithoutVAT;

                    order.SalesDiscount = Helpers.ToDouble(orderNode.SelectSingleNode("column [@columnName='OrderSalesDiscount']").InnerText);
                }
                catch (Exception ex)
                {
                }
                //Set OrderCustomFields values
                if (order.OrderFieldValues.Count > 0)
                {
                    XmlNode fieldNode = null;
                    foreach (OrderFieldValue ofv in order.OrderFieldValues)
                    {
                        fieldNode = orderNode.SelectSingleNode(string.Format("column [@columnName='{0}']", ofv.OrderField.SystemName));
                        if (fieldNode != null)
                        {
                            ofv.Value = fieldNode.InnerText;
                        }
                    }
                }
                var discountOrderLines = new OrderLineCollection();
                var discountInfoList = new List<DiscountInfo>();
                var taxList = new List<Tax>();

                XmlNodeList orderLinesNodes = response.SelectNodes("//item [@table='EcomOrderLines']");
                if (orderLinesNodes != null && orderLinesNodes.Count > 0)//Process OrderLines
                {
                    List<string> orderlineIDs = new List<string>();
                    List<OrderLine> orderLines = order.OrderLines.ToList();
                    OrderLineFieldCollection allOrderLineFields = OrderLineField.GetOrderLineFields();

                    foreach (XmlNode orderLineNode in orderLinesNodes)
                    {
                        string orderLineType =
                            orderLineNode.SelectSingleNode("column [@columnName='OrderLineType']").InnerText;
                        if (orderLineType == "0" || string.IsNullOrEmpty(orderLineType))
                        {
                            string productNumber =
                                orderLineNode.SelectSingleNode("column [@columnName='OrderLineProductNumber']").
                                    InnerText;

                            if (!string.IsNullOrEmpty(productNumber))
                            {
                                OrderLine orderLine = orderLines.FirstOrDefault(ol => ol.ProductNumber == productNumber);
                                if (orderLine != null)
                                {
                                    //Remove found line for getting next line with same ProductNumber
                                    orderLines.Remove(orderLine);
                                }
                                else
                                {
                                    // Create an OrderLine if it doesn't exist
                                    orderLine = CreateOrderLine(order, productNumber);
                                    if (orderLine == null)
                                        continue;
                                }
                                if (!string.IsNullOrEmpty(orderLine.Id))
                                    orderlineIDs.Add(orderLine.Id);
                                // Set standard values on OrderLines
                                orderLine.AllowOverridePrices = true;
                                orderLine.Quantity = Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLineQuantity']").InnerText);
                                string unitId = orderLineNode.SelectSingleNode("column [@columnName='OrderLineUnitID']")?.InnerText;
                                if (!string.IsNullOrWhiteSpace(unitId))
                                {
                                    orderLine.UnitId = unitId;
                                }

                                PriceInfo unitPrice = new PriceInfo();
                                unitPrice.PriceWithVAT = Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLineUnitPriceWithVAT']").InnerText);
                                unitPrice.PriceWithoutVAT = Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLineUnitPriceWithoutVAT']").InnerText);
                                orderLine.SetUnitPrice(unitPrice, false);
                                orderLine.Type = Convert.ToString(((int)OrderLineType.Fixed));
                                orderLine.Price.Currency = Helpers.CurrentCurrency;
                                orderLine.Price.PriceWithVAT = Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLinePriceWithVAT']").InnerText);
                                orderLine.Price.PriceWithoutVAT = Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLinePriceWithoutVAT']").InnerText);
                                //Set OrderLineCustomFields values                                
                                if (allOrderLineFields != null && allOrderLineFields.Count > 0)
                                {
                                    XmlNode orderLineFieldNode = null;
                                    foreach (OrderLineField olf in allOrderLineFields)
                                    {
                                        orderLineFieldNode = orderLineNode.SelectSingleNode(string.Format("column [@columnName='{0}']", olf.SystemName));
                                        if (orderLineFieldNode != null)
                                        {
                                            if (orderLine.OrderLineFieldValues == null)
                                            {
                                                orderLine.OrderLineFieldValues = new OrderLineFieldValueCollection();
                                            }
                                            OrderLineFieldValue olfv = orderLine.OrderLineFieldValues.FirstOrDefault(fv => fv != null && string.Compare(olf.SystemName, fv.OrderLineFieldSystemName, true) == 0);
                                            if (olfv != null)
                                            {
                                                olfv.Value = orderLineFieldNode.InnerText;
                                            }
                                            else
                                            {
                                                olfv = new OrderLineFieldValue(olf.SystemName, orderLineFieldNode.InnerText);
                                                orderLine.OrderLineFieldValues.Add(olfv);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (orderLineType == "1" || orderLineType == "3") //1=order discount, 3=Product Discount
                        {
                            DiscountInfo info = new DiscountInfo();
                            info.OrderLineType = Convert.ToString(((int)OrderLineType.ProductDiscount));
                            info.Quantity = Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLineQuantity']").InnerText); ;
                            info.PriceWithVAT = -Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLinePriceWithVAT']").InnerText);
                            info.PriceWithoutVAT = -Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLinePriceWithoutVAT']").InnerText);
                            info.VAT = -Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLinePriceVAT']").InnerText);

                            if (orderLineType == "3") //1=order discount, 3=Product Discount
                            {
                                string parentProductID = orderLineNode.SelectSingleNode("column [@columnName='OrderLineProductNumber']").InnerText;
                                foreach (var productOrderLine in order.OrderLines)
                                {
                                    if (productOrderLine.ProductId == parentProductID)
                                        info.ParentProductID = parentProductID;
                                }
                            }
                            discountInfoList.Add(info);
                            var orderLine = LiveSalesDiscountProvider.GetOrderLine(info, order);
                            discountOrderLines.Add(orderLine);
                        }
                        if (orderLineType == "4") //4=Tax order line
                        {
                            taxList.Add(GetTax(order, orderLineNode));
                        }
                    }
                    // Remove deleted OrderLines
                    for (int i = order.OrderLines.Count - 1; i >= 0; i--)
                    {
                        OrderLine orderline = order.OrderLines[i];
                        if (string.IsNullOrEmpty(orderline.Id))
                            continue;
                        if (!orderlineIDs.Contains(orderline.Id))
                        {
                            order.OrderLines.Remove(orderline);
                        }
                    }
                }
                //if (createOrder)
                //{
                foreach (var discountOrderLine in discountOrderLines)
                    order.OrderLines.Add(discountOrderLine);
                XmlNode intgrationIdNode = orderNode.SelectSingleNode("column [@columnName='OrderID']");
                if (intgrationIdNode != null && !string.IsNullOrEmpty(intgrationIdNode.InnerText))
                {
                    order.IntegrationOrderId = intgrationIdNode.InnerText;
                }
                //}                

                if (createOrder)
                {
                    string orderStateID = Helpers.GetConfigValue(Constants.OrderSettings.Path, Constants.OrderSettings.OrderStateAfterExport);
                    bool isOrderCreated = orderNode.SelectSingleNode("column [@columnName='OrderCreated']") != null &&
                        bool.Parse(orderNode.SelectSingleNode("column [@columnName='OrderCreated']").InnerText);
                    if (!isOrderCreated)
                    {
                        RemoveDiscounts(order);
                        order.OrderLines.Add(discountOrderLines);
                        Logger.Instance.Log(ErrorLevel.Error, string.Format("Order with ID '{0}' was not created in the ERP system.", order.Id));
                        HttpContext.Current.Session["LiveIntegration.OrderExportFailed"] = true;
                        HttpContext.Current.Session["LiveIntegration.FailedOrderID"] = order.Id;
                        order.Complete = false;
                        orderStateID = Helpers.GetConfigValue(Constants.OrderSettings.Path, Constants.OrderSettings.OrderStateAfterExportFailed);
                    }
                    else
                    {
                        HttpContext.Current.Session["LiveIntegration.OrderExportFailed"] = null;
                        HttpContext.Current.Session["LiveIntegration.FailedOrderID"] = null;
                    }
                    if (!string.IsNullOrEmpty(orderStateID))
                    {
                        order.StateId = orderStateID;
                    }
                    order.Save();
                }

                //add discount orderlines to cache, to be added by the discountProvider                
                HttpContext.Current.Session["LiveIntegrationDiscountsList" + order.Id] = discountInfoList;
                //add taxes list to cache, to be added by the tax Provider                
                HttpContext.Current.Session["LiveIntegrationTaxList" + order.Id] = taxList;
            }
            catch (Exception e)
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Response does not match schema: '{0}'.", e.Message));
            }
        }

        private static void RemoveDiscounts(Order order)
        {
            foreach (var orderLine in order.OrderLines)
            {
                if (orderLine.Type == "3" || orderLine.Type == "1")
                {
                    order.OrderLines.Remove(orderLine);
                }
            }
        }

        private static OrderLine CreateOrderLine(Order order, string productId)
        {
            var scalar = Database.ExecuteScalar(string.Format("SELECT TOP 1 ProductID FROM EcomProducts WHERE ProductNumber = '{0}'", productId));
            if (scalar == null || string.IsNullOrEmpty(scalar.ToString()))
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Can not CreateOrderLine: No product found with ProductID = '{0}'", productId));
                return null;
            }
            var product = Products.Product.GetProductById(scalar.ToString());
            if (product == null || product.Id != productId)
            {
                Logger.Instance.Log(ErrorLevel.Error,
                    string.Format("Can not CreateOrderLine: No product found or ProductNumber is different from the response ProductID = '{0}'", productId));
                return null;
            }
            OrderLine orderLine = new OrderLine();
            orderLine.SetProductInformation(product);
            orderLine.Type = ((int)OrderLineType.Product).ToString();
            orderLine.Order = order;
            order.OrderLines.Add(orderLine);
            return orderLine;
        }

        private static Tax GetTax(Order order, XmlNode orderLineNode)
        {
            Tax tax = new Tax();
            //set it to false if the Dynamicweb VAT addition is not needed
            tax.CalculateVat = true;
            if (orderLineNode.SelectSingleNode("column [@columnName='OrderLineTaxAmount']") != null)
            {
                tax.Amount = new PriceRaw(Helpers.ToDouble(orderLineNode.SelectSingleNode("column [@columnName='OrderLineTaxAmount']").InnerText),
                    order.Currency);
            }
            string parentProductNumber = string.Empty;
            if (orderLineNode.SelectSingleNode("column [@columnName='OrderLineProductNumber']") != null)
            {
                parentProductNumber = orderLineNode.SelectSingleNode("column [@columnName='OrderLineProductNumber']").InnerText;
            }
            if (!string.IsNullOrEmpty(parentProductNumber))
            {
                foreach (var productOrderLine in order.OrderLines)
                {
                    if (productOrderLine.ProductNumber == parentProductNumber)
                    {
                        tax.Product = productOrderLine.Product;
                        break;
                    }
                }
            }
            return tax;
        }
    }
}
